{ PVCAM Demonstration Script

  This script removes "hot" pixels from a long exposure.
  It assumes that hot pixels are those that are more than
  95% the intensity of the maximum pixel intensity in the image.

  The script also outputs the number of hot pixels, and optionally
  lists the coordinates of the hot pixels.

  The script PVDEMO06.V adds hot pixels to an existing image
  for testing purposes.

  Algorithm:
  1. Scan for image maximum.
  2. Set threshold at (say) 95% of maximum.
  3. Create binary image which is 1 wherever a pixel exceeds the
     threshold, and 0 elsewhere.
  4. Smooth a *copy* of the image with a median filter.
  5. Multiply the smoothed image by the binary image to create
     the "patch-up" image.
  6. Multiply the original image by the inverse of the binary
     image to zero the hot pixels.
  7. Replace the old hot pixels with the smoothed pixels.

  Notes:
  1. Some experimentation with the threshold may be necessary.
  2. For large clusters of hot pixels several passes of the
     median filter may be required.
  3. The 3x3 median filter will miss hot pixels on the boundary
     of the image. A simple solution would be to copy the original
     image into a slightly larger image before filtering, then
     copy the relevant portion back again. }

const
  ChooseMsg = 'Select an image with hot pixels' ;
  ErrorMsg  = 'This script requires a desktop image' ;
  QuitMsg   = 'No hot pixels were detected' ;
  QueryMsg  = ' hot pixels were detected.;Do you want a list of coordinates?' ;
  MemoryMsg = 'Too many hot pixel coordinates to list' ;

  Threshold = 0.95 ;

var
  Maximum ;     { Maximum pixel intensity in HotImage }
  HotThresh ;   { Threshold pixel intensity }
  HotMask ;     { Binary image of hot pixels }

  HotImage ;    { Hot-pixel image to correct }
  PatchImage ;  { Patch-up image }
  NewImage ;    { Corrected image }

  nHot ;        { The number of hot pixels }
  index ;       { Looping index }
  HotCoords ;   { Hot pixel coordinates }


begin

{ Check that a desktop image exists }
if GetNumImages = 0 then Halt( ErrorMsg ) ;

{ Select the hot image }
SelectImage( ChooseMsg,HotImage ) ;

{ Scan for maximum and set threshold level }
Maximum := MaxOf( HotImage ) ;
HotThresh := word( Threshold * Maximum ) ;

{ Create hot mask }
HotMask := HotImage >= HotThresh ;

{ Count number of hot pixels }
nHot := SumOf( HotMask ) ;

{ Quit if no hot pixels found }
if nHot = 0 then Halt( QuitMsg ) ;

{ Create smooth version of hot image }
PatchImage := RankFilter( HotImage,5 ) ;

{ Mask out pixels that are ok in smooth image }
PatchImage := PatchImage * HotMask ;

{ Zero the hot pixels in original }
NewImage := HotImage ;
NewImage := NewImage * not HotMask ;

{ Combine original and smoothed images }
NewImage := NewImage + PatchImage ;

{ Display result }
Show( NewImage,'newimage' ) ;

{ Ask if hot coordinates are required }
if Query( nHot,QueryMsg ) = id_Yes then
  begin

  { Find locations }
  HotCoords := Find( HotMask ) ;

  { Check there was enough memory for Find() since
    arrays must be less than 32767x32767 }
  if IsNull( HotCoords ) then
    Halt( MemoryMsg )
  else
    begin
    Writeln( 'A total of ',nHot,' hot pixels were detected.' ) ;
    Writeln ;
    Writeln( 'Coordinates:' ) ;
    for index := 0 to GetYSize( HotCoords )-1 do
      Writeln( HotCoords[ 0,index ],',',HotCoords[ 1,index ] ) ;
    end ;

  end ;

end
